/*
 * @author Mário Soares
 * @contributors Jefferson González
 *
 * @license
 * This file is part of wxPHP check the LICENSE file for information.
 *
 * @description
 * Manual binding implemetation of the wxApp class
 *
*/

#include "php_wxwidgets.h"
#include "app.h"

/**
 * Set the wxWidgets application handler.
 */
IMPLEMENT_APP_NO_MAIN(wxAppWrapper);

BEGIN_EXTERN_C()
void php_wxApp_free(void *object TSRMLS_DC)
{
    #ifdef USE_WXPHP_DEBUG
    php_printf("Calling php_wxApp_free on %s at line %i\n", zend_get_executed_filename(TSRMLS_C), zend_get_executed_lineno(TSRMLS_C));
    php_printf("===========================================\n");
    #endif

    zo_wxApp* custom_object = (zo_wxApp*) object;
    //delete custom_object->native_object;
    zend_object_std_dtor(&custom_object->zo TSRMLS_CC);
    efree(custom_object);
}

zend_object_value php_wxApp_new(zend_class_entry *class_type TSRMLS_DC)
{
    #ifdef USE_WXPHP_DEBUG
    php_printf("Calling php_wxApp_new on %s at line %i\n", zend_get_executed_filename(TSRMLS_C), zend_get_executed_lineno(TSRMLS_C));
    php_printf("===========================================\n");
    #endif

    zend_object_value retval;
    zo_wxApp* custom_object;
    custom_object = (zo_wxApp*) emalloc(sizeof(zo_wxApp));

    zend_object_std_init(&custom_object->zo, class_type TSRMLS_CC);

    #if PHP_VERSION_ID < 50399
    zval *temp;
    ALLOC_HASHTABLE(custom_object->zo.properties);
    zend_hash_init(custom_object->zo.properties, 0, NULL, ZVAL_PTR_DTOR, 0);
    zend_hash_copy(custom_object->zo.properties, &class_type->default_properties, (copy_ctor_func_t) zval_add_ref,(void *) &temp, sizeof(zval *));
    #else
    object_properties_init(&custom_object->zo, class_type);
    #endif

    custom_object->native_object = NULL;
    custom_object->object_type = PHP_WXAPP_TYPE;
    custom_object->is_user_initialized = 0;

    retval.handle = zend_objects_store_put(custom_object, NULL, php_wxApp_free, NULL TSRMLS_CC);
    retval.handlers = zend_get_std_object_handlers();

    return retval;
}
END_EXTERN_C()

int wxAppWrapper::OnExit()
{
    zval *retval;
    zval func_name;

    ZVAL_STRINGL(&func_name, (char *)"OnExit", sizeof("OnExit")-1, 0);

    call_user_function_ex(NULL, &phpObj, &func_name, &retval, 0, NULL, 0, NULL TSRMLS_CC);

    return 0;
}

bool wxAppWrapper::OnInit()
{
    zval *retval;
    zval func_name;

    ZVAL_STRINGL(&func_name, (char *)"OnInit", sizeof("OnInit")-1, 0);

    wxFileSystem::AddHandler(new wxZipFSHandler);

    call_user_function_ex(NULL, &phpObj, &func_name, &retval, 0, NULL, 0, NULL TSRMLS_CC);

    return true;
}

#ifdef __WXMAC__
void wxAppWrapper::MacNewFile()
{
    static bool is_php_user_space_implemented = true;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Invoking virtual MacNewFile\n");
    php_printf("===========================================\n");
    #endif

    zval function_name;
    ZVAL_STRINGL(&function_name, (char *)"MacNewFile", sizeof("MacNewFile")-1, 0);

    zval* return_value;
    MAKE_STD_ZVAL(return_value);

    int function_called;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Trying to call user defined method\n");
    #endif

    if(is_php_user_space_implemented)
    {
        function_called = call_user_function_ex(NULL, &phpObj, &function_name, &return_value, 0, NULL, 0, NULL TSRMLS_CC);
    }
    else
    {
        function_called = FAILURE;
    }

    if(function_called == FAILURE)
    {
        is_php_user_space_implemented = false;

        #ifdef USE_WXPHP_DEBUG
        php_printf("Invocation of user defined method failed\n");
        #endif
    }
}

void wxAppWrapper::MacOpenFiles ( const wxArrayString &  fileNames)
{
    static bool is_php_user_space_implemented = true;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Invoking virtual MacOpenFiles\n");
    php_printf("===========================================\n");
    #endif

    zval function_name;
    ZVAL_STRINGL(&function_name, (char *)"MacOpenFiles", sizeof("MacOpenFiles")-1, 0);

    zval** params[1];
    zval *arguments[1];

    //Initilize arguments array
    MAKE_STD_ZVAL(arguments[0]);

    zval* return_value;
    MAKE_STD_ZVAL(return_value);

    char* temp_string;
    int function_called;

    //Parameters for conversion
    array_init(arguments[0]);
    for(int i=0; i<fileNames.GetCount(); i++)
    {
        temp_string = (char*)malloc(sizeof(wxChar)*(fileNames[i].size()+1));
        strcpy(temp_string, (const char *) fileNames[i].char_str());
        add_next_index_string(arguments[0], temp_string, 1);
        free(temp_string);
    }

    //Set parameters that are sent to function
    params[0] = &arguments[0];

    #ifdef USE_WXPHP_DEBUG
    php_printf("Trying to call user defined method\n");
    #endif

    if(is_php_user_space_implemented)
    {
        function_called = call_user_function_ex(NULL, &phpObj, &function_name, &return_value, 1, params, 0, NULL TSRMLS_CC);
    }
    else
    {
        function_called = FAILURE;
    }

    //Delete already used parameters from memory
    zval_ptr_dtor(&arguments[0]);

    if(function_called == FAILURE)
    {
        is_php_user_space_implemented = false;

        #ifdef USE_WXPHP_DEBUG
        php_printf("Invocation of user defined method failed\n");
        #endif
    }
}

void wxAppWrapper::MacOpenFile(const wxString& fileName)
{
    static bool is_php_user_space_implemented = true;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Invoking virtual MacOpenFile\n");
    php_printf("===========================================\n");
    #endif

    zval function_name;
    ZVAL_STRINGL(&function_name, (char *)"MacOpenFile", sizeof("MacOpenFile")-1, 0);

    zval** params[1];
    zval *arguments[1];

    //Initilize arguments array
    MAKE_STD_ZVAL(arguments[0]);

    zval* return_value;
    MAKE_STD_ZVAL(return_value);

    char* temp_string;
    int function_called;

    //Parameters for conversion
    temp_string = (char*)malloc(sizeof(wxChar)*(fileName.size()+1));
    strcpy(temp_string, (const char *) fileName.char_str());
    ZVAL_STRING(arguments[0], temp_string, 1);
    free(temp_string);

    //Set parameters that are sent to function
    params[0] = &arguments[0];

    #ifdef USE_WXPHP_DEBUG
    php_printf("Trying to call user defined method\n");
    #endif

    if(is_php_user_space_implemented)
    {
        function_called = call_user_function_ex(NULL, &phpObj, &function_name, &return_value, 1, params, 0, NULL TSRMLS_CC);
    }
    else
    {
        function_called = FAILURE;
    }

    //Delete already used parameters from memory
    zval_ptr_dtor(&arguments[0]);

    if(function_called == FAILURE)
    {
        is_php_user_space_implemented = false;

        #ifdef USE_WXPHP_DEBUG
        php_printf("Invocation of user defined method failed\n");
        #endif
    }
}

void wxAppWrapper::MacOpenURL(const wxString& url)
{
    static bool is_php_user_space_implemented = true;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Invoking virtual MacOpenURL\n");
    php_printf("===========================================\n");
    #endif

    zval function_name;
    ZVAL_STRINGL(&function_name, (char *)"MacOpenURL", sizeof("MacOpenURL")-1, 0);

    zval** params[1];
    zval *arguments[1];

    //Initilize arguments array
    MAKE_STD_ZVAL(arguments[0]);

    zval* return_value;
    MAKE_STD_ZVAL(return_value);

    char* temp_string;
    int function_called;

    //Parameters for conversion
    temp_string = (char*)malloc(sizeof(wxChar)*(url.size()+1));
    strcpy(temp_string, (const char *) url.char_str());
    ZVAL_STRING(arguments[0], temp_string, 1);
    free(temp_string);

    //Set parameters that are sent to function
    params[0] = &arguments[0];

    #ifdef USE_WXPHP_DEBUG
    php_printf("Trying to call user defined method\n");
    #endif

    if(is_php_user_space_implemented)
    {
        function_called = call_user_function_ex(NULL, &phpObj, &function_name, &return_value, 1, params, 0, NULL TSRMLS_CC);
    }
    else
    {
        function_called = FAILURE;
    }

    //Delete already used parameters from memory
    zval_ptr_dtor(&arguments[0]);

    if(function_called == FAILURE)
    {
        is_php_user_space_implemented = false;

        #ifdef USE_WXPHP_DEBUG
        php_printf("Invocation of user defined method failed\n");
        #endif
    }
}

void wxAppWrapper::MacPrintFile(const wxString& fileName)
{
    static bool is_php_user_space_implemented = true;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Invoking virtual MacPrintFile\n");
    php_printf("===========================================\n");
    #endif

    zval function_name;
    ZVAL_STRINGL(&function_name, (char *)"MacPrintFile", sizeof("MacPrintFile")-1, 0);

    zval** params[1];
    zval *arguments[1];

    //Initilize arguments array
    MAKE_STD_ZVAL(arguments[0]);

    zval* return_value;
    MAKE_STD_ZVAL(return_value);

    char* temp_string;
    int function_called;

    //Parameters for conversion
    temp_string = (char*)malloc(sizeof(wxChar)*(fileName.size()+1));
    strcpy(temp_string, (const char *) fileName.char_str());
    ZVAL_STRING(arguments[0], temp_string, 1);
    free(temp_string);

    //Set parameters that are sent to function
    params[0] = &arguments[0];

    #ifdef USE_WXPHP_DEBUG
    php_printf("Trying to call user defined method\n");
    #endif

    if(is_php_user_space_implemented)
    {
        function_called = call_user_function_ex(NULL, &phpObj, &function_name, &return_value, 1, params, 0, NULL TSRMLS_CC);
    }
    else
    {
        function_called = FAILURE;
    }

    //Delete already used parameters from memory
    zval_ptr_dtor(&arguments[0]);

    if(function_called == FAILURE)
    {
        is_php_user_space_implemented = false;

        #ifdef USE_WXPHP_DEBUG
        php_printf("Invocation of user defined method failed\n");
        #endif
    }
}

void wxAppWrapper::MacReopenApp()
{
    static bool is_php_user_space_implemented = true;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Invoking virtual MacReopenApp\n");
    php_printf("===========================================\n");
    #endif

    zval function_name;
    ZVAL_STRINGL(&function_name, (char *)"MacReopenApp", sizeof("MacReopenApp")-1, 0);

    zval* return_value;
    MAKE_STD_ZVAL(return_value);

    int function_called;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Trying to call user defined method\n");
    #endif

    if(is_php_user_space_implemented)
    {
        function_called = call_user_function_ex(NULL, &phpObj, &function_name, &return_value, 0, NULL, 0, NULL TSRMLS_CC);
    }
    else
    {
        function_called = FAILURE;
    }

    if(function_called == FAILURE)
    {
        is_php_user_space_implemented = false;

        #ifdef USE_WXPHP_DEBUG
        php_printf("Invocation of user defined method failed\n");
        #endif
    }
}

bool wxAppWrapper::OSXIsGUIApplication()
{
    static bool is_php_user_space_implemented = true;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Invoking virtual OSXIsGUIApplication\n");
    php_printf("===========================================\n");
    #endif

    zval function_name;
    ZVAL_STRINGL(&function_name, (char *)"OSXIsGUIApplication", sizeof("OSXIsGUIApplication")-1, 0);

    zval* return_value;
    MAKE_STD_ZVAL(return_value);

    int function_called;

    #ifdef USE_WXPHP_DEBUG
    php_printf("Trying to call user defined method\n");
    #endif

    if(is_php_user_space_implemented)
    {
        function_called = call_user_function_ex(NULL, &phpObj, &function_name, &return_value, 0, NULL, 0, NULL TSRMLS_CC);
    }
    else
    {
        function_called = FAILURE;
    }

    if(function_called == FAILURE)
    {
        is_php_user_space_implemented = false;

        #ifdef USE_WXPHP_DEBUG
        php_printf("Invocation of user defined method failed\n");
        #endif

        return true;
    }

    #ifdef USE_WXPHP_DEBUG
    php_printf("Returning userspace value.\n");
    #endif

    return Z_BVAL_P(return_value);
}
#endif

/* {{{ proto wxApp wxApp::__construct()
   Constructor. */
PHP_METHOD(php_wxApp, __construct)
{
    zo_wxApp* current_object;

    wxAppWrapper* native_object = new wxAppWrapper();
    native_object->phpObj = getThis();

    #ifdef ZTS
    native_object->tsrm_ls = tsrm_ls;
    #endif

    current_object = (zo_wxApp*) zend_object_store_get_object(getThis() TSRMLS_CC);

    current_object->native_object = native_object;

    current_object->is_user_initialized = 1;
}
/* }}} */

/* {{{ proto void wxApp::SetInstance(wxApp app)
   Allows external code to modify global wxTheApp, but you should really know what you're doing if you call it.*/
PHP_METHOD(php_wxApp, SetInstance)
{
    zval *objvar;

    if(zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, (char *)"O", &objvar, php_wxApp_entry) == FAILURE)
    {
        RETURN_NULL();
    }

    wxApp::SetInstance((wxAppWrapper*) ((zo_wxApp*) zend_object_store_get_object(objvar TSRMLS_CC))->native_object);
}
/* }}} */

/* {{{ proto bool wxApp::Yield() */
PHP_METHOD(php_wxApp, Yield)
{
    wxAppWrapper* native_object = ((zo_wxApp*) zend_object_store_get_object(getThis() TSRMLS_CC))->native_object;

    if (zend_parse_parameters_none() == FAILURE)
        return;

    RETURN_BOOL(((wxApp*)native_object)->Yield())
}
/* }}} */
